import { Directive, Input } from '@angular/core';
import { area, curveLinear } from 'd3-shape';
import { scaleTime, scaleLinear } from 'd3-scale';
import { select } from 'd3-selection';
import { ChartUtils } from '../chart.utils';
import * as i0 from "@angular/core";
import * as i1 from "../chart.component";
import * as i2 from "../chart.utils";
export class AreaDirective {
    constructor(_elRef, _chart, _chartUtils) {
        this._elRef = _elRef;
        this._chart = _chart;
        this._chartUtils = _chartUtils;
        /**
         * Each data entry may have 2 or 3 values. If there are 3 the area is drawn within the bounds.
         *
         * If there are 2 the area is drawn to the chart height
         */
        this.data = [];
        this.animDuration = ChartUtils.ANIMATION_DURATION;
        this.curve = curveLinear;
        this.easing = ChartUtils.ANIMATION_EASING;
        this.xScale = scaleTime();
        this.yScale = scaleLinear();
    }
    ngOnInit() {
        this.areaSelection = select(this._elRef.nativeElement);
        this.setDomains();
        this.setArea();
        this.drawArea();
        this._subscribeToChartResize();
    }
    ngOnChanges(changes) {
        const isDomainChange = (changes.xDomain || changes.yDomain) && ChartUtils.haveDomainsChanged(changes.xDomain, changes.yDomain);
        const isDataChange = changes.data && !changes.data.firstChange && !ChartUtils.areDatasetsEqual(changes.data.previousValue, changes.data.currentValue);
        const isCurveChange = changes.curve && !changes.curve.firstChange && (changes.curve.currentValue !== changes.curve.previousValue);
        if (isDomainChange || isDataChange || isCurveChange) {
            this.setDomains();
            this.setArea();
            this.updateArea();
        }
    }
    setDomains() {
        this.xScale.domain(this.xDomain).range([0, this._chart.width]);
        this.yScale.domain(this.yDomain).range([this._chart.height, 0]);
    }
    setArea() {
        this.area = area()
            .x(d => this.xScale(d[0]))
            .y0(d => d[2] ? this.yScale(d[2]) : this._chart.height)
            .y1(d => this.yScale(d[1]));
    }
    drawArea() {
        this.areaSelection
            .datum(this.data)
            .attr('d', this.area);
    }
    updateArea() {
        this.areaSelection
            .datum(this.data)
            .transition()
            .duration(this.animDuration)
            .ease(this.easing)
            .attr('d', this.area);
    }
    _subscribeToChartResize() {
        this._chartResizeSub = this._chartUtils.chartResize$
            .subscribe(_ => {
            this.setDomains();
            this.setArea();
            this.drawArea();
        });
    }
    ngOnDestroy() {
        this._chartResizeSub.unsubscribe();
    }
}
AreaDirective.ɵfac = function AreaDirective_Factory(t) { return new (t || AreaDirective)(i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i1.ChartComponent), i0.ɵɵdirectiveInject(i2.ChartUtils)); };
AreaDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: AreaDirective, selectors: [["path", "nw-area", ""]], inputs: { data: [i0.ɵɵInputFlags.None, "nw-area", "data"], xDomain: "xDomain", yDomain: "yDomain", animDuration: "animDuration", curve: "curve", easing: "easing", xScale: "xScale" }, exportAs: ["nw-area"], features: [i0.ɵɵNgOnChangesFeature] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(AreaDirective, [{
        type: Directive,
        args: [{
                // eslint-disable-next-line @angular-eslint/directive-selector
                selector: 'path[nw-area]',
                exportAs: 'nw-area'
            }]
    }], () => [{ type: i0.ElementRef }, { type: i1.ChartComponent }, { type: i2.ChartUtils }], { data: [{
            type: Input,
            args: ['nw-area']
        }], xDomain: [{
            type: Input
        }], yDomain: [{
            type: Input
        }], animDuration: [{
            type: Input
        }], curve: [{
            type: Input
        }], easing: [{
            type: Input
        }], xScale: [{
            type: Input
        }] }); })();
