import { Directive, Input, Output, EventEmitter } from '@angular/core';
import { axisBottom, axisTop } from 'd3-axis';
import { AxisBase } from './axis-base';
import { scaleTime } from 'd3-scale';
import * as i0 from "@angular/core";
import * as i1 from "../chart.component";
import * as i2 from "../chart.utils";
export class XAxisDirective extends AxisBase {
    constructor(elRef, chart, chartUtils) {
        super(elRef, chart, chartUtils);
        this.align = 'bottom';
        this.scale = scaleTime();
        this.scaleUpdated = new EventEmitter();
    }
    createAxis() {
        this.axis = this.align === "bottom" ?
            axisBottom(this.scale) :
            axisTop(this.scale);
    }
    setTicks() {
        super.setTicks();
        if (this.showGuidlines) {
            this.axis.tickSizeInner(-this.chart.height);
        }
        else {
            this.axis.tickSizeInner(6);
        }
        this.scaleUpdated.emit(this.scale.copy());
    }
    setDomain() {
        this.scale.domain(this.domain).range([0, this.chart.width]);
    }
    positionLabel() {
        if (this.label) {
            const y = this.align === 'top' ? 0 : this.fullHeight;
            const dy = this.align === 'top' ? "1em" : "-0.5em";
            this.axisLabelSelection
                .attr('class', 'axis-label ' + this.align)
                .attr('dy', dy)
                .attr('transform', `translate(${this.fullWidth / 2}, ${y})`)
                .text(this.label);
        }
    }
    render() {
        const yTranslation = this._getAxisTranslation();
        this.axisSelection
            .attr('transform', "translate(0," + yTranslation + ")")
            .call(this.axis);
        this.positionLabel();
    }
    _getAxisTranslation() {
        if (this.align === 'bottom') {
            return this.chart.height;
        }
        if (this.align === 'center') {
            return this.chart.height / 2;
        }
        return 0;
    }
}
XAxisDirective.ɵfac = function XAxisDirective_Factory(t) { return new (t || XAxisDirective)(i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i1.ChartComponent), i0.ɵɵdirectiveInject(i2.ChartUtils)); };
XAxisDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: XAxisDirective, selectors: [["", "nw-x-axis", ""]], inputs: { align: "align", domain: "domain", scale: "scale" }, outputs: { scaleUpdated: "scaleUpdated" }, exportAs: ["nw-x-axis"], features: [i0.ɵɵInheritDefinitionFeature] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(XAxisDirective, [{
        type: Directive,
        args: [{
                // eslint-disable-next-line @angular-eslint/directive-selector
                selector: '[nw-x-axis]',
                exportAs: 'nw-x-axis'
            }]
    }], () => [{ type: i0.ElementRef }, { type: i1.ChartComponent }, { type: i2.ChartUtils }], { align: [{
            type: Input
        }], domain: [{
            type: Input
        }], scale: [{
            type: Input
        }], scaleUpdated: [{
            type: Output
        }] }); })();
