import { Directive, Input, Output, EventEmitter } from '@angular/core';
import { axisLeft, axisRight } from 'd3-axis';
import { AxisBase } from './axis-base';
import { scaleLinear } from 'd3-scale';
import * as i0 from "@angular/core";
import * as i1 from "../chart.component";
import * as i2 from "../chart.utils";
export class YAxisDirective extends AxisBase {
    constructor(elRef, chart, chartUtils) {
        super(elRef, chart, chartUtils);
        this.align = 'left';
        this.scaleUpdated = new EventEmitter();
        this.scale = scaleLinear();
    }
    createAxis() {
        this.axis = this.align === "left" ?
            axisLeft(this.scale) :
            axisRight(this.scale);
    }
    setTicks() {
        super.setTicks();
        if (this.showGuidlines) {
            this.axis.tickSizeInner(-this.chart.width);
        }
        else {
            this.axis.tickSizeInner(6);
        }
        this.scaleUpdated.emit(this.scale.copy());
    }
    setDomain() {
        this.scale.domain(this.domain).range([this.chart.height, 0]);
    }
    positionLabel() {
        if (this.label) {
            const rotationAngle = this.align === 'left' ? '-90' : '90';
            const y = this.align === 'left' ? 0 : -this.fullWidth;
            const x = this.align === 'left' ? -(this.fullHeight / 2) : (this.fullHeight / 2);
            this.axisLabelSelection
                .attr('class', 'axis-label ' + this.align)
                .attr('transform', `rotate(${rotationAngle})`)
                .attr('y', y)
                .attr('x', x)
                .attr('dy', "1em")
                .text(this.label);
        }
    }
    render() {
        const xTranslation = this._getAxisTranslation();
        this.axisSelection
            .attr('transform', "translate(" + xTranslation + ", 0)")
            .call(this.axis);
        this.positionLabel();
    }
    _getAxisTranslation() {
        if (this.align === 'right') {
            return this.chart.width;
        }
        if (this.align === 'center') {
            return this.chart.width / 2;
        }
        return 0;
    }
}
YAxisDirective.ɵfac = function YAxisDirective_Factory(t) { return new (t || YAxisDirective)(i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i1.ChartComponent), i0.ɵɵdirectiveInject(i2.ChartUtils)); };
YAxisDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: YAxisDirective, selectors: [["", "nw-y-axis", ""]], inputs: { align: "align", domain: "domain" }, outputs: { scaleUpdated: "scaleUpdated" }, exportAs: ["nw-y-axis"], features: [i0.ɵɵInheritDefinitionFeature] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(YAxisDirective, [{
        type: Directive,
        args: [{
                // eslint-disable-next-line @angular-eslint/directive-selector
                selector: '[nw-y-axis]',
                exportAs: 'nw-y-axis'
            }]
    }], () => [{ type: i0.ElementRef }, { type: i1.ChartComponent }, { type: i2.ChartUtils }], { align: [{
            type: Input
        }], domain: [{
            type: Input
        }], scaleUpdated: [{
            type: Output
        }] }); })();
