import { Directive, Input, Output, EventEmitter } from '@angular/core';
import { select } from 'd3-selection';
import { brush, brushX, brushY } from 'd3-brush';
import * as i0 from "@angular/core";
import * as i1 from "../chart.component";
export class BrushDirective {
    constructor(_elRef, _chart) {
        this._elRef = _elRef;
        this._chart = _chart;
        this.dimension = '';
        this.brushEnd = new EventEmitter();
    }
    ngOnInit() {
        this.initialize();
        this._subscribeToBrushEndEvent();
    }
    ngOnChanges(changes) {
        if (changes.dimension && !changes.dimension.firstChange && changes.dimension.previousValue !== changes.dimension.currentValue) {
            this.reset();
        }
    }
    initialize() {
        this.brushSelection = select(this._elRef.nativeElement);
        this.brushSelection.attr('class', 'nw-brush');
        this._createBrush();
    }
    reset() {
        this._unbindEvents();
        this._clearBrush();
        this._createBrush();
        this._subscribeToBrushEndEvent();
    }
    _createBrush() {
        switch (this.dimension) {
            case 'x':
                this.brush = brushX();
                break;
            case 'y':
                this.brush = brushY();
                break;
            default:
                this.brush = brush();
                break;
        }
        this.brush.extent(this.getExtent());
        this.brushSelection.call(this.brush);
    }
    getExtent() {
        return this.extent || [
            [0, 0],
            [this._chart.width, this._chart.height]
        ];
    }
    _subscribeToBrushEndEvent() {
        this.brush.on('end', () => {
            const selection = this.brushSelection.select('rect.selection');
            const nw = [+selection.attr('x'), +selection.attr('y')];
            const se = [
                +selection.attr('x') + +selection.attr('width'),
                +selection.attr('y') + +selection.attr('height')
            ];
            const emission = nw[0] === se[0] && nw[1] === se[1] ?
                null :
                [nw, se];
            this.brushEnd.emit(emission);
        });
    }
    _clearBrush() {
        this.brush.move(this.brushSelection, null);
        this.brushEnd.emit(null);
    }
    _unbindEvents() {
        this.brush.on('end', null);
    }
    ngOnDestroy() {
        this._unbindEvents();
    }
}
BrushDirective.ɵfac = function BrushDirective_Factory(t) { return new (t || BrushDirective)(i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i1.ChartComponent)); };
BrushDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: BrushDirective, selectors: [["g", "nw-brush", ""]], inputs: { extent: "extent", dimension: "dimension" }, outputs: { brushEnd: "brushEnd" }, exportAs: ["nw-brush"], features: [i0.ɵɵNgOnChangesFeature] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(BrushDirective, [{
        type: Directive,
        args: [{
                // eslint-disable-next-line @angular-eslint/directive-selector
                selector: 'svg:g[nw-brush]',
                exportAs: 'nw-brush'
            }]
    }], () => [{ type: i0.ElementRef }, { type: i1.ChartComponent }], { extent: [{
            type: Input
        }], dimension: [{
            type: Input
        }], brushEnd: [{
            type: Output
        }] }); })();
