import { Component, Input, ViewChild, ChangeDetectionStrategy, Output, EventEmitter } from '@angular/core';
import { select, pointer } from 'd3-selection';
import { ChartUtils } from './chart.utils';
import { fromEvent, Subject, merge } from 'rxjs';
import { debounceTime, takeUntil } from 'rxjs/operators';
import * as i0 from "@angular/core";
import * as i1 from "./chart.utils";
import * as i2 from "../resize-observer/resize-observer.directive";
const _c0 = ["chartContainer"];
const _c1 = ["mouseEventCaptureContainer"];
const _c2 = ["hoverOverlay"];
const _c3 = ["nw-chart", ""];
const _c4 = [[["", "nw-x-axis", ""], ["", "nw-y-axis", ""]], [["", 8, "nw-slot-1"]], [["", 8, "nw-slot-2"]], "*"];
const _c5 = ["[nw-x-axis],[nw-y-axis]", ".nw-slot-1", ".nw-slot-2", "*"];
export class ChartComponent {
    constructor(_elRef, chartUtils) {
        this._elRef = _elRef;
        this.chartUtils = chartUtils;
        this.margins = { top: 0, bottom: 0, left: 0, right: 0 };
        this.nwMousemove = new EventEmitter();
        this.nwMouseleave = new EventEmitter();
        this.bgClick = new EventEmitter();
        this._destroyed$ = new Subject();
        this._chartDimensionsChange$ = new Subject();
    }
    ngOnInit() {
        this.parentElement = this._elRef.nativeElement.parentElement;
        this.setSvg();
        this.setHoverOverlay();
        this.subscribeToChartResize();
    }
    ngAfterViewInit() {
        this.appendContainer();
    }
    get width() {
        return (this.w || this._elRef.nativeElement.getBoundingClientRect().width) - this.margins.left - this.margins.right;
    }
    get height() {
        return (this.h || this._elRef.nativeElement.getBoundingClientRect().height) - this.margins.top - this.margins.bottom;
    }
    setSvg() {
        this.svg = select(this._elRef.nativeElement)
            .attr("width", this.width + this.margins.left + this.margins.right)
            .attr("height", this.height + this.margins.top + this.margins.bottom);
    }
    appendContainer() {
        select(this.chartContainer.nativeElement)
            .attr("transform", "translate(" + this.margins.left + "," + this.margins.top + ")");
    }
    setHoverOverlay() {
        const self = this;
        // This full height / width rect ensures that all mouse events will
        // be captured and delegated to the parent mouseEventCaptureContainer
        select(this.hoverOverlay.nativeElement)
            .attr("width", this.width)
            .attr("height", this.height)
            .style('fill', 'none')
            .style('pointer-events', 'all');
        select(this.mouseEventCaptureContainer.nativeElement)
            .style('pointer-events', 'all')
            .on('mouseleave', () => this.nwMouseleave.emit())
            .on('mousemove', e => {
            // emits the current mouse position
            self.nwMousemove.emit(pointer(e));
        });
    }
    onBackgroundClick() {
        this.bgClick.emit();
    }
    /**
     * As well charts being resized due to the window being resized, another scenario is that
     * the parent element is squeezed or stretched in a reponsive design due to the appearance
     * or disappearance of other elements. For this we use our nwResizeObserver which utilizes
     * ResizeObserver under the hood.
     */
    subscribeToChartResize() {
        const windowResize$ = fromEvent(window, 'resize');
        merge(windowResize$, this._chartDimensionsChange$)
            .pipe(debounceTime(100), takeUntil(this._destroyed$))
            .subscribe(_ => this.chartUtils.notifyChartResize());
    }
    onChartDimensionsChange() {
        this._chartDimensionsChange$.next();
    }
    ngOnDestroy() {
        this._destroyed$.next();
        this._destroyed$.complete();
    }
}
ChartComponent.ɵfac = function ChartComponent_Factory(t) { return new (t || ChartComponent)(i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i1.ChartUtils)); };
ChartComponent.ɵcmp = /*@__PURE__*/ i0.ɵɵdefineComponent({ type: ChartComponent, selectors: [["svg", "nw-chart", ""]], viewQuery: function ChartComponent_Query(rf, ctx) { if (rf & 1) {
        i0.ɵɵviewQuery(_c0, 7);
        i0.ɵɵviewQuery(_c1, 7);
        i0.ɵɵviewQuery(_c2, 7);
    } if (rf & 2) {
        let _t;
        i0.ɵɵqueryRefresh(_t = i0.ɵɵloadQuery()) && (ctx.chartContainer = _t.first);
        i0.ɵɵqueryRefresh(_t = i0.ɵɵloadQuery()) && (ctx.mouseEventCaptureContainer = _t.first);
        i0.ɵɵqueryRefresh(_t = i0.ɵɵloadQuery()) && (ctx.hoverOverlay = _t.first);
    } }, inputs: { w: [i0.ɵɵInputFlags.None, "width", "w"], h: [i0.ɵɵInputFlags.None, "height", "h"], margins: "margins" }, outputs: { nwMousemove: "nwMousemove", nwMouseleave: "nwMouseleave", bgClick: "bgClick" }, exportAs: ["nw-chart"], features: [i0.ɵɵProvidersFeature([ChartUtils])], attrs: _c3, ngContentSelectors: _c5, decls: 10, vars: 1, consts: [["chartContainer", ""], ["mouseEventCaptureContainer", ""], ["hoverOverlay", ""], ["nwResizeObserver", "", 3, "nwResize", "element"], [1, "mouse-event-capture-container"], [3, "click"]], template: function ChartComponent_Template(rf, ctx) { if (rf & 1) {
        const _r1 = i0.ɵɵgetCurrentView();
        i0.ɵɵprojectionDef(_c4);
        i0.ɵɵnamespaceSVG();
        i0.ɵɵelementStart(0, "g", 3, 0);
        i0.ɵɵlistener("nwResize", function ChartComponent_Template_g_nwResize_0_listener() { i0.ɵɵrestoreView(_r1); return i0.ɵɵresetView(ctx.onChartDimensionsChange()); });
        i0.ɵɵprojection(2);
        i0.ɵɵelementStart(3, "g", 4, 1);
        i0.ɵɵprojection(5, 1);
        i0.ɵɵelementStart(6, "rect", 5, 2);
        i0.ɵɵlistener("click", function ChartComponent_Template_rect_click_6_listener() { i0.ɵɵrestoreView(_r1); return i0.ɵɵresetView(ctx.onBackgroundClick()); });
        i0.ɵɵelementEnd();
        i0.ɵɵprojection(8, 2);
        i0.ɵɵprojection(9, 3);
        i0.ɵɵelementEnd()();
    } if (rf & 2) {
        i0.ɵɵproperty("element", ctx.parentElement);
    } }, dependencies: [i2.ResizeObserverDirective], encapsulation: 2, changeDetection: 0 });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(ChartComponent, [{
        type: Component,
        args: [{
                selector: 'svg[nw-chart]',
                template: `
        <svg:g #chartContainer
            nwResizeObserver
            [element]="parentElement"
            (nwResize)="onChartDimensionsChange()">
            <!-- Keep axes outside the main rendering container so
            that we only get mouse events from graphical elements -->
            <ng-content select="[nw-x-axis],[nw-y-axis]"></ng-content>
            <svg:g #mouseEventCaptureContainer class="mouse-event-capture-container">
                <ng-content select=".nw-slot-1"></ng-content>
                <svg:rect #hoverOverlay (click)="onBackgroundClick()"></svg:rect>
                <ng-content select=".nw-slot-2"></ng-content>
                <ng-content></ng-content>
            </svg:g>
        </svg:g>
    `,
                providers: [ChartUtils],
                exportAs: 'nw-chart',
                changeDetection: ChangeDetectionStrategy.OnPush
            }]
    }], () => [{ type: i0.ElementRef }, { type: i1.ChartUtils }], { w: [{
            type: Input,
            args: ['width']
        }], h: [{
            type: Input,
            args: ['height']
        }], margins: [{
            type: Input
        }], chartContainer: [{
            type: ViewChild,
            args: ['chartContainer', { static: true }]
        }], mouseEventCaptureContainer: [{
            type: ViewChild,
            args: ['mouseEventCaptureContainer', { static: true }]
        }], hoverOverlay: [{
            type: ViewChild,
            args: ['hoverOverlay', { static: true }]
        }], nwMousemove: [{
            type: Output
        }], nwMouseleave: [{
            type: Output
        }], bgClick: [{
            type: Output
        }] }); })();
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassDebugInfo(ChartComponent, { className: "ChartComponent" }); })();
