import { Directive, Input, Output, EventEmitter } from '@angular/core';
import { select } from 'd3-selection';
import { line, curveLinear } from 'd3-shape';
import { scaleTime, scaleLinear } from 'd3-scale';
import { ChartUtils } from '../chart.utils';
import 'd3-transition';
import * as i0 from "@angular/core";
import * as i1 from "../chart.component";
import * as i2 from "../chart.utils";
export class PathDirective {
    constructor(_elRef, _chart, _chartUtils) {
        this._elRef = _elRef;
        this._chart = _chart;
        this._chartUtils = _chartUtils;
        this.data = [];
        this.animDuration = ChartUtils.ANIMATION_DURATION;
        this.curve = curveLinear;
        this.easing = ChartUtils.ANIMATION_EASING;
        this.xScale = scaleTime();
        this.animEnd = new EventEmitter();
        this.yScale = scaleLinear();
    }
    ngOnInit() {
        this.path = select(this._elRef.nativeElement);
        this.setDomains();
        this.setLine();
        this.drawLine();
        this._subscribeToChartResize();
    }
    ngOnChanges(changes) {
        const isDomainChange = (changes.xDomain || changes.yDomain) && ChartUtils.haveDomainsChanged(changes.xDomain, changes.yDomain);
        const isDataChange = changes.data && !changes.data.firstChange && !ChartUtils.areDatasetsEqual(changes.data.previousValue, changes.data.currentValue);
        const isCurveChange = changes.curve && !changes.curve.firstChange && (changes.curve.currentValue !== changes.curve.previousValue);
        if (isDomainChange || isDataChange || isCurveChange) {
            this.setDomains();
            this.setLine();
            this.updateLine();
        }
    }
    setDomains() {
        this.xScale.domain(this.xDomain).range([0, this._chart.width]);
        this.yScale.domain(this.yDomain).range([this._chart.height, 0]);
    }
    setLine() {
        this.line = line().curve(this.curve)
            .x(d => this.xScale(d[0]))
            .y(d => this.yScale(d[1]));
    }
    drawLine() {
        this.path
            .datum(this.data)
            .attr('d', this.line);
    }
    updateLine() {
        this.path
            .datum(this.data)
            .transition()
            .duration(this.animDuration)
            .ease(this.easing)
            .attr('d', this.line)
            .on('end', e => this.animEnd.next());
    }
    _subscribeToChartResize() {
        this._chartResizeSub = this._chartUtils.chartResize$
            .subscribe(_ => {
            this.setDomains();
            this.drawLine();
        });
    }
    ngOnDestroy() {
        this._chartResizeSub.unsubscribe();
    }
}
PathDirective.ɵfac = function PathDirective_Factory(t) { return new (t || PathDirective)(i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i1.ChartComponent), i0.ɵɵdirectiveInject(i2.ChartUtils)); };
PathDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: PathDirective, selectors: [["path", "nw-path", ""]], inputs: { data: [i0.ɵɵInputFlags.None, "nw-path", "data"], xDomain: "xDomain", yDomain: "yDomain", animDuration: "animDuration", curve: "curve", easing: "easing", xScale: "xScale" }, outputs: { animEnd: "animEnd" }, exportAs: ["nw-path"], features: [i0.ɵɵNgOnChangesFeature] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(PathDirective, [{
        type: Directive,
        args: [{
                // eslint-disable-next-line @angular-eslint/directive-selector
                selector: 'path[nw-path]',
                exportAs: 'nw-path'
            }]
    }], () => [{ type: i0.ElementRef }, { type: i1.ChartComponent }, { type: i2.ChartUtils }], { data: [{
            type: Input,
            args: ['nw-path']
        }], xDomain: [{
            type: Input
        }], yDomain: [{
            type: Input
        }], animDuration: [{
            type: Input
        }], curve: [{
            type: Input
        }], easing: [{
            type: Input
        }], xScale: [{
            type: Input
        }], animEnd: [{
            type: Output
        }] }); })();
