import { Directive, Input } from '@angular/core';
import { ChartUtils } from '../chart.utils';
import { select } from 'd3-selection';
import { scaleTime, scaleLinear } from 'd3-scale';
import * as i0 from "@angular/core";
import * as i1 from "../chart.component";
import * as i2 from "../chart.utils";
export class TextDirective {
    constructor(_elRef, _chart, _chartUtils) {
        this._elRef = _elRef;
        this._chart = _chart;
        this._chartUtils = _chartUtils;
        this.animDuration = ChartUtils.ANIMATION_DURATION;
        this.easing = ChartUtils.ANIMATION_EASING;
        this.xScale = scaleTime();
        this.yScale = scaleLinear();
    }
    ngOnInit() {
        this.text = select(this._elRef.nativeElement);
        this.setDomains();
        this.draw();
        this._subscribeToChartResize();
    }
    ngOnChanges(changes) {
        const isDomainChange = (changes.xDomain || changes.yDomain) && ChartUtils.haveDomainsChanged(changes.xDomain, changes.yDomain);
        const isDataChange = changes.point && !changes.point.firstChange && !ChartUtils.areDatasetsEqual([changes.point.previousValue], [changes.point.currentValue]);
        if (isDomainChange || isDataChange) {
            this.setDomains();
            this.update();
        }
    }
    setDomains() {
        this.xScale.domain(this.xDomain).range([0, this._chart.width]);
        this.yScale.domain(this.yDomain).range([this._chart.height, 0]);
    }
    draw() {
        this.text
            .attr("x", this.x)
            .attr("y", this.y);
    }
    update() {
        this.text
            .transition()
            .duration(this.animDuration)
            .ease(this.easing)
            .attr("x", this.x)
            .attr("y", this.y);
    }
    get x() {
        return this.xScale(this.point[0]);
    }
    get y() {
        return this.yScale(this.point[1]);
    }
    _subscribeToChartResize() {
        this._chartResizeSub = this._chartUtils.chartResize$
            .subscribe(_ => {
            this.setDomains();
            this.draw();
        });
    }
    ngOnDestroy() {
        this._chartResizeSub.unsubscribe();
    }
}
TextDirective.ɵfac = function TextDirective_Factory(t) { return new (t || TextDirective)(i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i1.ChartComponent), i0.ɵɵdirectiveInject(i2.ChartUtils)); };
TextDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: TextDirective, selectors: [["text", "nw-text", ""]], inputs: { point: [i0.ɵɵInputFlags.None, "nw-text", "point"], xDomain: "xDomain", yDomain: "yDomain", animDuration: "animDuration", easing: "easing", xScale: "xScale" }, exportAs: ["nw-text"], features: [i0.ɵɵNgOnChangesFeature] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(TextDirective, [{
        type: Directive,
        args: [{
                // eslint-disable-next-line @angular-eslint/directive-selector
                selector: 'text[nw-text]',
                exportAs: 'nw-text'
            }]
    }], () => [{ type: i0.ElementRef }, { type: i1.ChartComponent }, { type: i2.ChartUtils }], { point: [{
            type: Input,
            args: ['nw-text']
        }], xDomain: [{
            type: Input
        }], yDomain: [{
            type: Input
        }], animDuration: [{
            type: Input
        }], easing: [{
            type: Input
        }], xScale: [{
            type: Input
        }] }); })();
