import { Directive, Input } from '@angular/core';
import * as i0 from "@angular/core";
export class ChartTooltipDirective {
    constructor(_elRef, _renderer) {
        this._elRef = _elRef;
        this._renderer = _renderer;
        this.offsetX = 16;
    }
    ngAfterViewInit() {
        this._setTooltipSize();
        this._renderer.setStyle(this._elRef.nativeElement, 'pointer-events', 'none');
        this._renderer.setStyle(this._elRef.nativeElement, 'position', 'absolute');
    }
    ngOnChanges(changes) {
        if (changes.position && changes.position.previousValue !== changes.position.currentValue) {
            const offsetLeft = changes.position.currentValue[0] + this.chartMargins.left;
            const offsetTop = changes.position.currentValue[1] + this.chartMargins.top;
            const isFlippedX = offsetLeft > this.chartWidth + this.chartMargins.left + this.chartMargins.right - this.offsetX - this._width;
            const isFlippedY = offsetTop > this.chartHeight + this.chartMargins.top + this.chartMargins.bottom - this._height;
            const x = isFlippedX ? `calc(${offsetLeft - this.offsetX}px - 100%)` : `${offsetLeft + this.offsetX}px`;
            const y = isFlippedY ? `calc(${offsetTop}px - 100%)` : `${offsetTop}px`;
            this._renderer.setStyle(this._elRef.nativeElement, 'transform', `translate(${x}, ${y})`);
        }
    }
    _setTooltipSize() {
        if (this._width === undefined || this._height === undefined) {
            const rect = this._elRef.nativeElement.getBoundingClientRect();
            this._width = rect.width;
            this._height = rect.height;
        }
    }
}
ChartTooltipDirective.ɵfac = function ChartTooltipDirective_Factory(t) { return new (t || ChartTooltipDirective)(i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i0.Renderer2)); };
ChartTooltipDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: ChartTooltipDirective, selectors: [["", "nwChartTooltip", ""]], inputs: { position: "position", chartWidth: "chartWidth", chartHeight: "chartHeight", chartMargins: "chartMargins", offsetX: "offsetX" }, features: [i0.ɵɵNgOnChangesFeature] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(ChartTooltipDirective, [{
        type: Directive,
        args: [{
                selector: '[nwChartTooltip]'
            }]
    }], () => [{ type: i0.ElementRef }, { type: i0.Renderer2 }], { position: [{
            type: Input
        }], chartWidth: [{
            type: Input
        }], chartHeight: [{
            type: Input
        }], chartMargins: [{
            type: Input
        }], offsetX: [{
            type: Input
        }] }); })();
