import { Directive, HostListener, ContentChildren } from '@angular/core';
import { DropdownDirective } from "./dropdown.directive";
import { filter, map, takeUntil } from "rxjs/operators";
import { merge, Subject } from 'rxjs';
import * as i0 from "@angular/core";
import * as i1 from "./dropdown.service";
export class DropdownMenuDirective {
    constructor(_service, _element, _cdRef) {
        this._service = _service;
        this._element = _element;
        this._cdRef = _cdRef;
        this._destroyed$ = new Subject();
    }
    ngAfterContentInit() {
        this._subscribeToChildrenVisbilityToggle();
        this._subscribeToScrollPosition();
    }
    _subscribeToScrollPosition() {
        this._service.toggle$.pipe(filter(opened => opened), takeUntil(this._destroyed$)).subscribe(() => {
            this._cdRef.detectChanges();
            this._scrollToActiveElement();
        });
    }
    _scrollToActiveElement() {
        const menuElement = this._element.nativeElement;
        const activeElement = this._element.nativeElement.querySelector('.active');
        if (activeElement) {
            menuElement.scrollTo({ top: activeElement.offsetTop });
        }
    }
    // When a sub dropdown menu is opened, force close any sibling sub dropdown menus
    _subscribeToChildrenVisbilityToggle() {
        const openEvents = this.nestedDropdowns
            .map((nd, index) => {
            return nd.opened
                .pipe(map(x => index));
        });
        merge(...openEvents).pipe(takeUntil(this._destroyed$)).subscribe(index => {
            // Close all other sibling dropdowns
            this.nestedDropdowns
                .filter((nd, i) => i !== index)
                .forEach(nd => nd.close());
        });
    }
    onClick(event) {
        if (this._shouldCloseOnMenuClick()) {
            return this._service.close();
        }
    }
    _shouldCloseOnMenuClick() {
        return this._service.autoClose === true || this._service.autoClose === 'inside';
    }
    ngOnDestroy() {
        this._destroyed$.next();
        this._destroyed$.complete();
    }
}
DropdownMenuDirective.ɵfac = function DropdownMenuDirective_Factory(t) { return new (t || DropdownMenuDirective)(i0.ɵɵdirectiveInject(i1.DropdownService), i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i0.ChangeDetectorRef)); };
DropdownMenuDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: DropdownMenuDirective, selectors: [["", "nwDropdownMenu", ""]], contentQueries: function DropdownMenuDirective_ContentQueries(rf, ctx, dirIndex) { if (rf & 1) {
        i0.ɵɵcontentQuery(dirIndex, DropdownDirective, 4);
    } if (rf & 2) {
        let _t;
        i0.ɵɵqueryRefresh(_t = i0.ɵɵloadQuery()) && (ctx.nestedDropdowns = _t);
    } }, hostBindings: function DropdownMenuDirective_HostBindings(rf, ctx) { if (rf & 1) {
        i0.ɵɵlistener("click", function DropdownMenuDirective_click_HostBindingHandler($event) { return ctx.onClick($event); });
    } }, exportAs: ["nw-dropdown-menu"] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(DropdownMenuDirective, [{
        type: Directive,
        args: [{
                selector: '[nwDropdownMenu]',
                exportAs: 'nw-dropdown-menu'
            }]
    }], () => [{ type: i1.DropdownService }, { type: i0.ElementRef }, { type: i0.ChangeDetectorRef }], { nestedDropdowns: [{
            type: ContentChildren,
            args: [DropdownDirective]
        }], onClick: [{
            type: HostListener,
            args: ['click', ['$event']]
        }] }); })();
