import { Directive, Input, HostListener } from '@angular/core';
import { debounceTime, filter, tap } from "rxjs/operators";
import { fromEvent } from 'rxjs';
import * as i0 from "@angular/core";
import * as i1 from "./dropdown.service";
export class DropdownToggleDirective {
    constructor(_service, _elRef, _cdRef) {
        this._service = _service;
        this._elRef = _elRef;
        this._cdRef = _cdRef;
        this.nwTrigger = "click";
        this.breakpoint = 767;
        this.isOpen = false;
        this._isMousingOver = false;
        this._mouseEventSubscriptions = [];
    }
    ngOnInit() {
        this._subscribeToToggle();
        if (this.nwTrigger === 'hover') {
            const mouseEnterSub = fromEvent(this._elRef.nativeElement, 'mouseenter')
                .pipe(filter(_ => !this._isMobileScreenSize()), tap(_ => this._isMousingOver = true), debounceTime(300), filter(_ => this._isMousingOver))
                .subscribe(event => this._open());
            const mouseLeaveSub = fromEvent(this._elRef.nativeElement, 'mouseleave')
                .pipe(filter(_ => !this._isMobileScreenSize()), tap(_ => this._isMousingOver = false), debounceTime(300), filter(_ => !this._isMousingOver))
                .subscribe(event => this._close());
            this._mouseEventSubscriptions = [mouseEnterSub, mouseLeaveSub];
        }
    }
    _isMobileScreenSize() {
        return window.innerWidth < this.breakpoint;
    }
    _subscribeToToggle() {
        this._toggleSub$ = this._service.toggle$.subscribe(isOpen => {
            this.isOpen = isOpen;
        });
    }
    _open() {
        this._isMousingOver = true;
        this._service.open();
        this._cdRef.detectChanges();
    }
    _close() {
        this._isMousingOver = false;
        this._service.close();
        this._cdRef.detectChanges();
    }
    toggle(event) {
        this._service.toggle();
        if (this.nwTrigger === 'hover') {
            event.stopImmediatePropagation();
        }
    }
    ngOnDestroy() {
        this._mouseEventSubscriptions.forEach(mes => mes.unsubscribe());
        this._toggleSub$.unsubscribe();
    }
}
DropdownToggleDirective.ɵfac = function DropdownToggleDirective_Factory(t) { return new (t || DropdownToggleDirective)(i0.ɵɵdirectiveInject(i1.DropdownService), i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i0.ChangeDetectorRef)); };
DropdownToggleDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: DropdownToggleDirective, selectors: [["", "nwDropdownToggle", ""]], hostAttrs: ["aria-haspopup", "true"], hostVars: 1, hostBindings: function DropdownToggleDirective_HostBindings(rf, ctx) { if (rf & 1) {
        i0.ɵɵlistener("click", function DropdownToggleDirective_click_HostBindingHandler($event) { return ctx.toggle($event); });
    } if (rf & 2) {
        i0.ɵɵattribute("aria-expanded", ctx.isOpen);
    } }, inputs: { nwTrigger: "nwTrigger", breakpoint: "breakpoint" }, exportAs: ["nw-dropdown-toggle"] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(DropdownToggleDirective, [{
        type: Directive,
        args: [{
                selector: '[nwDropdownToggle]',
                exportAs: 'nw-dropdown-toggle',
                host: {
                    'aria-haspopup': 'true',
                    '[attr.aria-expanded]': 'isOpen'
                }
            }]
    }], () => [{ type: i1.DropdownService }, { type: i0.ElementRef }, { type: i0.ChangeDetectorRef }], { nwTrigger: [{
            type: Input
        }], breakpoint: [{
            type: Input
        }], toggle: [{
            type: HostListener,
            args: ['click', ['$event']]
        }] }); })();
