import { Directive, Input, HostBinding, Output, EventEmitter } from '@angular/core';
import { DropdownService } from "./dropdown.service";
import { filter } from 'rxjs/operators';
import * as i0 from "@angular/core";
import * as i1 from "./dropdown.service";
export class DropdownDirective {
    constructor(_service, _elRef, _zone, _renderer, _cdRef) {
        this._service = _service;
        this._elRef = _elRef;
        this._zone = _zone;
        this._renderer = _renderer;
        this._cdRef = _cdRef;
        this.autoClose = true;
        this.elementsToIgnore = [];
        this.selectorsToIgnore = [];
        this.disabled = false;
        this.opened = new EventEmitter();
        this.closed = new EventEmitter();
    }
    ngOnInit() {
        this._service.autoClose = this.autoClose;
        this._subscribeToToggle();
        // For performance reasons, bind to document click outside the zone
        this._zone.runOutsideAngular(() => {
            // Store a reference to the "unlisten" functions returned from these methods
            // https://angular.io/api/core/Renderer2#listen
            this._documentUnlistener = this._renderer.listen('document', 'click', this.onDocumentClick.bind(this));
            this._escapeUnlistener = this._renderer.listen('document', 'keydown.escape', this.onEscape.bind(this));
        });
    }
    ngOnChanges(changes) {
        // autoClose is essentially an optional input. What this means is that if you don't pass a
        // value to the `autoClose` input the "if" check below will never evaluate to true. This
        // is why we don't just rely on ngOnChanges to set the value in the service, we also use
        // ngOnInit to set the initial value, with subsequent input changes handled below
        if (changes.autoClose && !changes.autoClose.firstChange) {
            this._service.autoClose = this.autoClose;
        }
    }
    open() {
        this._service.open();
    }
    close() {
        this._service.close();
    }
    _subscribeToToggle() {
        this._toggleSubscription = this._service.toggle$
            /**
             * If the dropdown is disabled, only allow close events through
             */
            .pipe(filter(isOpen => {
            if (this.disabled) {
                return !isOpen;
            }
            return true;
        }))
            .subscribe(isOpen => {
            this.isOpen = isOpen;
            if (this.isOpen) {
                this.opened.emit();
            }
            else {
                this.closed.emit();
            }
        });
    }
    onDocumentClick(event) {
        const shouldClose = (this.autoClose === true || this.autoClose === 'outside') && this.isOpen;
        // Don't bother evaluating the source of the event if `shouldClose` is false
        if (shouldClose) {
            const target = event.target;
            const containers = [this._elRef.nativeElement].concat(this.elementsToIgnore);
            const isEventSourceFromWithinDropdown = this._service.isHTMLElementContainedIn(target, containers) && !target.classList.contains('dropdown-backdrop');
            const isEventSourceFromWithinSelectors = this.selectorsToIgnore.length ?
                this._service.isHTMLElementInPath(event, this.selectorsToIgnore) :
                false;
            if (!isEventSourceFromWithinDropdown && !isEventSourceFromWithinSelectors) {
                this._zone.run(() => {
                    this._service.close();
                    this._cdRef.detectChanges();
                });
            }
        }
    }
    onEscape(event) {
        if (this.isOpen) {
            this._zone.run(() => {
                this._service.close();
                this._cdRef.detectChanges();
            });
        }
    }
    ngOnDestroy() {
        this._toggleSubscription.unsubscribe();
        if (this._documentUnlistener) {
            this._zone.runOutsideAngular(() => this._documentUnlistener());
        }
        if (this._escapeUnlistener) {
            this._zone.runOutsideAngular(() => this._escapeUnlistener());
        }
    }
}
DropdownDirective.ɵfac = function DropdownDirective_Factory(t) { return new (t || DropdownDirective)(i0.ɵɵdirectiveInject(i1.DropdownService), i0.ɵɵdirectiveInject(i0.ElementRef), i0.ɵɵdirectiveInject(i0.NgZone), i0.ɵɵdirectiveInject(i0.Renderer2), i0.ɵɵdirectiveInject(i0.ChangeDetectorRef)); };
DropdownDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: DropdownDirective, selectors: [["", "nwDropdown", ""]], hostVars: 4, hostBindings: function DropdownDirective_HostBindings(rf, ctx) { if (rf & 2) {
        i0.ɵɵclassProp("disabled", ctx.disabled)("open", ctx.isOpen);
    } }, inputs: { autoClose: "autoClose", elementsToIgnore: "elementsToIgnore", selectorsToIgnore: "selectorsToIgnore", disabled: "disabled" }, outputs: { opened: "opened", closed: "closed" }, exportAs: ["nw-dropdown"], features: [i0.ɵɵProvidersFeature([DropdownService]), i0.ɵɵNgOnChangesFeature] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(DropdownDirective, [{
        type: Directive,
        args: [{
                selector: '[nwDropdown]',
                exportAs: 'nw-dropdown',
                providers: [DropdownService]
            }]
    }], () => [{ type: i1.DropdownService }, { type: i0.ElementRef }, { type: i0.NgZone }, { type: i0.Renderer2 }, { type: i0.ChangeDetectorRef }], { autoClose: [{
            type: Input
        }], elementsToIgnore: [{
            type: Input
        }], selectorsToIgnore: [{
            type: Input
        }], disabled: [{
            type: HostBinding,
            args: ['class.disabled']
        }, {
            type: Input
        }], opened: [{
            type: Output
        }], closed: [{
            type: Output
        }], isOpen: [{
            type: HostBinding,
            args: ['class.open']
        }] }); })();
