import { Directive, Output, EventEmitter, Input } from "@angular/core";
import { Subject } from "rxjs";
import { skip, debounceTime, takeUntil, tap } from "rxjs/operators";
import * as i0 from "@angular/core";
export class ResizeObserverDirective {
    constructor(_elRef) {
        /**
         * A pixel value specifying the minimum change in height or width that triggers the resize event
         */
        this.tolerance = 0;
        this.nwResize = new EventEmitter();
        this._nwResize = new Subject();
        this._destroyed$ = new Subject();
        this.element = this.element || _elRef.nativeElement;
    }
    ngOnInit() {
        this.subscribeToResize();
        if (ResizeObserver) {
            this.createObserver();
        }
    }
    subscribeToResize() {
        /**
         * Rather than call this.nwResize.emit() immediately after being notified
         * of the resize event, we use this._nwResize as a proxy to filter events we
         * don't want and to debounce as we don't want to emit hundreds of events
         * if, for example, the width or height of the element is being animated
         */
        this._nwResize.asObservable().pipe(
        /**
         * Store the contentRect of the element so that we can compare changes
         */
        tap(entry => this._currentElementRect = entry.contentRect), 
        /**
         * use skip(1) because an event is emitting immediately
         * after calling .observe on the observer
         */
        skip(1), debounceTime(50), takeUntil(this._destroyed$)).subscribe(res => this.nwResize.emit());
    }
    createObserver() {
        this._observer = new ResizeObserver(([element]) => {
            var _a, _b, _c, _d;
            const widthChange = Math.abs(element.contentRect.width - ((_b = (_a = this._currentElementRect) === null || _a === void 0 ? void 0 : _a.width) !== null && _b !== void 0 ? _b : 0));
            const heightChange = Math.abs(element.contentRect.height - ((_d = (_c = this._currentElementRect) === null || _c === void 0 ? void 0 : _c.height) !== null && _d !== void 0 ? _d : 0));
            if ((widthChange >= this.tolerance) || (heightChange >= this.tolerance)) {
                this._nwResize.next(element);
            }
        });
        this._observer.observe(this.element);
    }
    ngOnDestroy() {
        if (ResizeObserver && this._observer) {
            this._observer.unobserve(this.element);
        }
        this._destroyed$.next();
        this._destroyed$.complete();
    }
}
ResizeObserverDirective.ɵfac = function ResizeObserverDirective_Factory(t) { return new (t || ResizeObserverDirective)(i0.ɵɵdirectiveInject(i0.ElementRef)); };
ResizeObserverDirective.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: ResizeObserverDirective, selectors: [["", "nwResizeObserver", ""]], inputs: { element: "element", tolerance: "tolerance" }, outputs: { nwResize: "nwResize" }, exportAs: ["nwResizeObserver"] });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(ResizeObserverDirective, [{
        type: Directive,
        args: [{
                selector: '[nwResizeObserver]',
                exportAs: 'nwResizeObserver'
            }]
    }], () => [{ type: i0.ElementRef }], { element: [{
            type: Input
        }], tolerance: [{
            type: Input
        }], nwResize: [{
            type: Output
        }] }); })();
