import { Injectable } from '@angular/core';
import { ComponentPortal, DomPortalOutlet } from '@angular/cdk/portal';
import { ToastsComponent } from './toasts.component';
import { defaultConfig } from './config';
import * as i0 from "@angular/core";
export class Toaster {
    constructor(_cfr, _appRef, _injector) {
        this._cfr = _cfr;
        this._appRef = _appRef;
        this._injector = _injector;
    }
    _attachOutlet() {
        /**
         * If the `ToastsComponent` already exists return it
         */
        if (this._toastsComponentRef) {
            return this._toastsComponentRef;
        }
        this._config = this._config || defaultConfig;
        this._toastPortal = new ComponentPortal(ToastsComponent);
        this._outlet = new DomPortalOutlet(this._config.outletElement, this._cfr, this._appRef, this._injector);
        this._toastsComponentRef = this._outlet.attach(this._toastPortal).instance;
        return this._toastsComponentRef;
    }
    show(toast) {
        this._attachOutlet();
        return this._toastsComponentRef.show(toast);
    }
    success(message) {
        this._attachOutlet();
        return this._toastsComponentRef.success(message);
    }
    error(message) {
        this._attachOutlet();
        return this._toastsComponentRef.error(message);
    }
    dismiss(toast) {
        this._toastsComponentRef.dismiss(toast);
    }
    isToastActive(toast) {
        return this._toastsComponentRef.getToastIndex(toast) > -1;
    }
    /**
     * Set the configuration values for a `Toaster` instance. This can only be called once
     * per `Toaster` instance and must be called before the outlet is created
     *
     * @param config Configuration settings for the Toaster
     */
    setConfig(config) {
        if (!this._config) {
            this._config = config;
        }
        else {
            console.warn(`config has already been set for this Toaster instance with a value of`, this._config);
        }
    }
}
Toaster.ɵfac = function Toaster_Factory(t) { return new (t || Toaster)(i0.ɵɵinject(i0.ComponentFactoryResolver), i0.ɵɵinject(i0.ApplicationRef), i0.ɵɵinject(i0.Injector)); };
Toaster.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: Toaster, factory: Toaster.ɵfac });
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(Toaster, [{
        type: Injectable
    }], () => [{ type: i0.ComponentFactoryResolver }, { type: i0.ApplicationRef }, { type: i0.Injector }], null); })();
