import { Overlay } from "@angular/cdk/overlay";
import { ElementRef, EventEmitter, Injector, OnChanges, OnDestroy, OnInit, SimpleChanges, TemplateRef, ViewContainerRef } from "@angular/core";
import { Placement } from "./models/Placement.type";
import * as i0 from "@angular/core";
export declare class TooltipDirective implements OnInit, OnChanges, OnDestroy {
    private _elRef;
    private _overlay;
    private _vcRef;
    private _injector;
    /**
     * This directive can be invoked by using the `nwTooltip` or `nwPopover` attributes. The only differences between using these
     * attributes are the default values of certain properties, e.g. `delay` and open and close events
     */
    tooltip: string | TemplateRef<any>;
    popover: string | TemplateRef<any>;
    /**
     * An object that can be passed when the `nwTooltip` or `nwPopover` input is a `TemplateRef`
     * ref: https://angular.io/api/core/ng-template#context
     */
    context: any;
    /**
     * One or more preferred placement options
     */
    placement: Placement | Placement[];
    /**
     * Manually control the opening and closing of the tooltip
     */
    isOpen: boolean;
    /**
     * When true, the tooltip will not not respond to any open or close events. Nor will it
     * respond to changes to the `isOpen` input
     */
    isDisabled: boolean;
    /**
     * Number of ms to wait before opening
     */
    delay: number;
    /**
     * Change the placement of the tooltip to its opposite position when it moves outside the viewport
     */
    autoFlip: boolean;
    /**
     * A list of events that open the tooltip
     */
    openEvents: string[];
    /**
     * A list of events that close the tooltip
     */
    closeEvents: string[];
    /**
     * A class to apply to the tooltip container
     */
    containerClass: string;
    /**
     * Display an arrow or not. The location of the arrow is dependant on the current `placement`
     */
    withArrow: boolean;
    /**
     * Display a close button or not
     */
    withClose: boolean;
    closeOnScroll: boolean;
    closeOnOutsideClick: boolean;
    /**
     * WARNING: Use with caution - there are potential performance issues with this
     *
     * Update the position of the tooltip before the next browser repaint. An example of where this may be required is if
     * the tooltip is attached (and open) to an element that transitions or animates to a new position
     */
    updatePositionOnAnimationFrame: boolean;
    /**
     * In the case where the tooltip should not be attached to the host element, a reference to another element can be used
     */
    connectedTo: ElementRef<HTMLElement> | Element;
    /**
     * Determines whether pointer events are enabled on the cdk-overlay-pane element
     */
    pointerEvents: 'auto' | 'none';
    /**
     * The screen size at which the tooltip should treated as a popover as there is no hover events on mobile
     */
    breakpoint: number;
    hostElementZIndex: number;
    nwShown: EventEmitter<null>;
    nwHidden: EventEmitter<null>;
    nwClose: EventEmitter<null>;
    private _overlayRef;
    private _destroyed$;
    private _tooltipArrowSize;
    private _manualToggleEvent$;
    private _cancelDelayedOpen$;
    /**
     * A subject that emits when the TooltipContainerComponent is destroyed
     */
    private _tooltipContainerDestroyed$;
    constructor(_elRef: ElementRef<HTMLElement>, _overlay: Overlay, _vcRef: ViewContainerRef, _injector: Injector);
    ngOnInit(): void;
    ngOnChanges(c: SimpleChanges): void;
    /**
     * Can be called manually from the exported directive to open the tooltip
     */
    show(): void;
    /**
     * Can be called manually from the exported directive to close the tooltip
     */
    hide(): void;
    /**
     * Can be called manually from the exported directive to toggle the tooltip
     */
    toggle(): void;
    /**
     * Based on the selector used, choose different default if inputs are not defined
     */
    private _setInputDefaults;
    private _getTooltipContent;
    private _open;
    private _close;
    private _createOverlay;
    /**
     * Create and return a custom injector that provides the tooltip text as an injectable dependency
     */
    private _createInjector;
    private _subscribeToEvents;
    private _getPositionPair;
    private _getPositionStrategy;
    private _updatePositionStrategy;
    private _getScrollStrategy;
    private _updateScrollStrategy;
    ngOnDestroy(): void;
    static ɵfac: i0.ɵɵFactoryDeclaration<TooltipDirective, never>;
    static ɵdir: i0.ɵɵDirectiveDeclaration<TooltipDirective, "[nwTooltip],[nwPopover]", ["nw-tooltip", "nw-popover"], { "tooltip": { "alias": "nwTooltip"; "required": false; }; "popover": { "alias": "nwPopover"; "required": false; }; "context": { "alias": "context"; "required": false; }; "placement": { "alias": "placement"; "required": false; }; "isOpen": { "alias": "isOpen"; "required": false; }; "isDisabled": { "alias": "isDisabled"; "required": false; }; "delay": { "alias": "delay"; "required": false; }; "autoFlip": { "alias": "autoFlip"; "required": false; }; "openEvents": { "alias": "openEvents"; "required": false; }; "closeEvents": { "alias": "closeEvents"; "required": false; }; "containerClass": { "alias": "containerClass"; "required": false; }; "withArrow": { "alias": "withArrow"; "required": false; }; "withClose": { "alias": "withClose"; "required": false; }; "closeOnScroll": { "alias": "closeOnScroll"; "required": false; }; "closeOnOutsideClick": { "alias": "closeOnOutsideClick"; "required": false; }; "updatePositionOnAnimationFrame": { "alias": "updatePositionOnAnimationFrame"; "required": false; }; "connectedTo": { "alias": "connectedTo"; "required": false; }; "pointerEvents": { "alias": "pointerEvents"; "required": false; }; "breakpoint": { "alias": "breakpoint"; "required": false; }; "hostElementZIndex": { "alias": "hostElementZIndex"; "required": false; }; }, { "nwShown": "nwShown"; "nwHidden": "nwHidden"; "nwClose": "nwClose"; }, never, never, false, never>;
}
